@@ -7,8 +7,20 @@ from .api_gateway_proxy_event import APIGatewayProxyEvent, APIGatewayProxyEventV
 from .appsync_resolver_event import AppSyncResolverEvent
 from .aws_config_rule_event import AWSConfigRuleEvent
 from .bedrock_agent_event import BedrockAgentEvent
+from .cloud_watch_alarm_event import (
+    CloudWatchAlarmConfiguration,
+    CloudWatchAlarmData,
+    CloudWatchAlarmEvent,
+    CloudWatchAlarmMetric,
+    CloudWatchAlarmMetricStat,
+    CloudWatchAlarmState,
+)
 from .cloud_watch_custom_widget_event import CloudWatchDashboardCustomWidgetEvent
 from .cloud_watch_logs_event import CloudWatchLogsEvent
+from .cloudformation_custom_resource_event import CloudFormationCustomResourceEvent
+from .code_deploy_lifecycle_hook_event import (
+    CodeDeployLifecycleHookEvent,
+)
 from .code_pipeline_job_event import CodePipelineJobEvent
 from .connect_contact_flow_event import ConnectContactFlowEvent
 from .dynamo_db_stream_event import DynamoDBStreamEvent
@@ -23,6 +35,11 @@ from .kinesis_firehose_event import (
 )
 from .kinesis_stream_event import KinesisStreamEvent
 from .lambda_function_url_event import LambdaFunctionUrlEvent
+from .s3_batch_operation_event import (
+    S3BatchOperationEvent,
+    S3BatchOperationResponse,
+    S3BatchOperationResponseRecord,
+)
 from .s3_event import S3Event, S3EventBridgeNotificationEvent
 from .secrets_manager_event import SecretsManagerEvent
 from .ses_event import SESEvent
@@ -37,8 +54,15 @@ __all__ = [
     "AppSyncResolverEvent",
     "ALBEvent",
     "BedrockAgentEvent",
+    "CloudWatchAlarmData",
+    "CloudWatchAlarmEvent",
+    "CloudWatchAlarmMetric",
+    "CloudWatchAlarmState",
+    "CloudWatchAlarmConfiguration",
+    "CloudWatchAlarmMetricStat",
     "CloudWatchDashboardCustomWidgetEvent",
     "CloudWatchLogsEvent",
+    "CodeDeployLifecycleHookEvent",
     "CodePipelineJobEvent",
     "ConnectContactFlowEvent",
     "DynamoDBStreamEvent",
@@ -52,6 +76,9 @@ __all__ = [
     "LambdaFunctionUrlEvent",
     "S3Event",
     "S3EventBridgeNotificationEvent",
+    "S3BatchOperationEvent",
+    "S3BatchOperationResponse",
+    "S3BatchOperationResponseRecord",
     "SESEvent",
     "SNSEvent",
     "SQSEvent",
@@ -59,4 +86,5 @@ __all__ = [
     "AWSConfigRuleEvent",
     "VPCLatticeEvent",
     "VPCLatticeEventV2",
+    "CloudFormationCustomResourceEvent",
 ]
