@@ -1,6 +1,9 @@
 """
 Persistence layers supporting idempotency
 """
+
+from __future__ import annotations
+
 import datetime
 import hashlib
 import json
@@ -8,105 +11,29 @@ import logging
 import os
 import warnings
 from abc import ABC, abstractmethod
-from types import MappingProxyType
-from typing import Any, Dict, Optional
+from typing import TYPE_CHECKING, Any
 
 import jmespath
 
 from aws_lambda_powertools.shared import constants
 from aws_lambda_powertools.shared.cache_dict import LRUDict
 from aws_lambda_powertools.shared.json_encoder import Encoder
-from aws_lambda_powertools.utilities.idempotency.config import IdempotencyConfig
 from aws_lambda_powertools.utilities.idempotency.exceptions import (
-    IdempotencyInvalidStatusError,
     IdempotencyItemAlreadyExistsError,
     IdempotencyKeyError,
     IdempotencyValidationError,
 )
+from aws_lambda_powertools.utilities.idempotency.persistence.datarecord import (
+    STATUS_CONSTANTS,
+    DataRecord,
+)
 from aws_lambda_powertools.utilities.jmespath_utils import PowertoolsFunctions
 
+if TYPE_CHECKING:
+    from aws_lambda_powertools.utilities.idempotency.config import IdempotencyConfig
+
 logger = logging.getLogger(__name__)
 
-STATUS_CONSTANTS = MappingProxyType({"INPROGRESS": "INPROGRESS", "COMPLETED": "COMPLETED", "EXPIRED": "EXPIRED"})
-
-
-class DataRecord:
-    """
-    Data Class for idempotency records.
-    """
-
-    def __init__(
-        self,
-        idempotency_key: str,
-        status: str = "",
-        expiry_timestamp: Optional[int] = None,
-        in_progress_expiry_timestamp: Optional[int] = None,
-        response_data: str = "",
-        payload_hash: str = "",
-    ) -> None:
-        """
-
-        Parameters
-        ----------
-        idempotency_key: str
-            hashed representation of the idempotent data
-        status: str, optional
-            status of the idempotent record
-        expiry_timestamp: int, optional
-            time before the record should expire, in seconds
-        in_progress_expiry_timestamp: int, optional
-            time before the record should expire while in the INPROGRESS state, in seconds
-        payload_hash: str, optional
-            hashed representation of payload
-        response_data: str, optional
-            response data from previous executions using the record
-        """
-        self.idempotency_key = idempotency_key
-        self.payload_hash = payload_hash
-        self.expiry_timestamp = expiry_timestamp
-        self.in_progress_expiry_timestamp = in_progress_expiry_timestamp
-        self._status = status
-        self.response_data = response_data
-
-    @property
-    def is_expired(self) -> bool:
-        """
-        Check if data record is expired
-
-        Returns
-        -------
-        bool
-            Whether the record is currently expired or not
-        """
-        return bool(self.expiry_timestamp and int(datetime.datetime.now().timestamp()) > self.expiry_timestamp)
-
-    @property
-    def status(self) -> str:
-        """
-        Get status of data record
-
-        Returns
-        -------
-        str
-        """
-        if self.is_expired:
-            return STATUS_CONSTANTS["EXPIRED"]
-        elif self._status in STATUS_CONSTANTS.values():
-            return self._status
-        else:
-            raise IdempotencyInvalidStatusError(self._status)
-
-    def response_json_as_dict(self) -> Optional[dict]:
-        """
-        Get response data deserialized to python dict
-
-        Returns
-        -------
-        Optional[dict]
-            previous response data deserialized
-        """
-        return json.loads(self.response_data) if self.response_data else None
-
 
 class BasePersistenceLayer(ABC):
     """
@@ -119,7 +46,7 @@ class BasePersistenceLayer(ABC):
         self.configured = False
         self.event_key_jmespath: str = ""
         self.event_key_compiled_jmespath = None
-        self.jmespath_options: Optional[dict] = None
+        self.jmespath_options: dict | None = None
         self.payload_validation_enabled = False
         self.validation_key_jmespath = None
         self.raise_on_no_idempotency_key = False
@@ -127,7 +54,7 @@ class BasePersistenceLayer(ABC):
         self.use_local_cache = False
         self.hash_function = hashlib.md5
 
-    def configure(self, config: IdempotencyConfig, function_name: Optional[str] = None) -> None:
+    def configure(self, config: IdempotencyConfig, function_name: str | None = None) -> None:
         """
         Initialize the base persistence layer from the configuration settings
 
@@ -161,13 +88,13 @@ class BasePersistenceLayer(ABC):
             self._cache = LRUDict(max_items=config.local_cache_max_items)
         self.hash_function = getattr(hashlib, config.hash_function)
 
-    def _get_hashed_idempotency_key(self, data: Dict[str, Any]) -> Optional[str]:
+    def _get_hashed_idempotency_key(self, data: dict[str, Any]) -> str | None:
         """
         Extract idempotency key and return a hashed representation
 
         Parameters
         ----------
-        data: Dict[str, Any]
+        data: dict[str, Any]
             Incoming data
 
         Returns
@@ -200,13 +127,13 @@ class BasePersistenceLayer(ABC):
             return False
         return not data
 
-    def _get_hashed_payload(self, data: Dict[str, Any]) -> str:
+    def _get_hashed_payload(self, data: dict[str, Any]) -> str:
         """
         Extract payload using validation key jmespath and return a hashed representation
 
         Parameters
         ----------
-        data: Dict[str, Any]
+        data: dict[str, Any]
             Payload
 
         Returns
@@ -238,16 +165,20 @@ class BasePersistenceLayer(ABC):
         hashed_data = self.hash_function(json.dumps(data, cls=Encoder, sort_keys=True).encode())
         return hashed_data.hexdigest()
 
-    def _validate_payload(self, data: Dict[str, Any], data_record: DataRecord) -> None:
+    def _validate_payload(
+        self,
+        data_payload: dict[str, Any] | DataRecord,
+        stored_data_record: DataRecord,
+    ) -> None:
         """
         Validate that the hashed payload matches data provided and stored data record
 
         Parameters
         ----------
-        data: Dict[str, Any]
+        data_payload: dict[str, Any] | DataRecord
             Payload
-        data_record: DataRecord
-            DataRecord instance
+        stored_data_record: DataRecord
+            DataRecord fetched from Dynamo or cache
 
         Raises
         ----------
@@ -256,8 +187,12 @@ class BasePersistenceLayer(ABC):
 
         """
         if self.payload_validation_enabled:
-            data_hash = self._get_hashed_payload(data=data)
-            if data_record.payload_hash != data_hash:
+            if isinstance(data_payload, DataRecord):
+                data_hash = data_payload.payload_hash
+            else:
+                data_hash = self._get_hashed_payload(data=data_payload)
+
+            if stored_data_record.payload_hash != data_hash:
                 raise IdempotencyValidationError("Payload does not match stored record for this event key")
 
     def _get_expiry_timestamp(self) -> int:
@@ -311,13 +246,13 @@ class BasePersistenceLayer(ABC):
         if idempotency_key in self._cache:
             del self._cache[idempotency_key]
 
-    def save_success(self, data: Dict[str, Any], result: dict) -> None:
+    def save_success(self, data: dict[str, Any], result: dict) -> None:
         """
         Save record of function's execution completing successfully
 
         Parameters
         ----------
-        data: Dict[str, Any]
+        data: dict[str, Any]
             Payload
         result: dict
             The response from function
@@ -345,15 +280,15 @@ class BasePersistenceLayer(ABC):
 
         self._save_to_cache(data_record=data_record)
 
-    def save_inprogress(self, data: Dict[str, Any], remaining_time_in_millis: Optional[int] = None) -> None:
+    def save_inprogress(self, data: dict[str, Any], remaining_time_in_millis: int | None = None) -> None:
         """
         Save record of function's execution being in progress
 
         Parameters
         ----------
-        data: Dict[str, Any]
+        data: dict[str, Any]
             Payload
-        remaining_time_in_millis: Optional[int]
+        remaining_time_in_millis: int | None
             If expiry of in-progress invocations is enabled, this will contain the remaining time available in millis
         """
 
@@ -370,11 +305,13 @@ class BasePersistenceLayer(ABC):
             payload_hash=self._get_hashed_payload(data=data),
         )
 
-        if remaining_time_in_millis:
+        # When Lambda kills the container after timeout, the remaining_time_in_millis is 0, which is considered False.
+        # Therefore, we need to check if remaining_time_in_millis is not None (>=0) to handle this case.
+        # See: https://github.com/aws-powertools/powertools-lambda-python/issues/4759
+        if remaining_time_in_millis is not None:
             now = datetime.datetime.now()
             period = datetime.timedelta(milliseconds=remaining_time_in_millis)
             timestamp = (now + period).timestamp()
-
             data_record.in_progress_expiry_timestamp = int(timestamp * 1000)
         else:
             warnings.warn(
@@ -390,13 +327,13 @@ class BasePersistenceLayer(ABC):
 
         self._put_record(data_record=data_record)
 
-    def delete_record(self, data: Dict[str, Any], exception: Exception):
+    def delete_record(self, data: dict[str, Any], exception: Exception):
         """
         Delete record from the persistence store
 
         Parameters
         ----------
-        data: Dict[str, Any]
+        data: dict[str, Any]
             Payload
         exception
             The exception raised by the function
@@ -418,13 +355,13 @@ class BasePersistenceLayer(ABC):
 
         self._delete_from_cache(idempotency_key=data_record.idempotency_key)
 
-    def get_record(self, data: Dict[str, Any]) -> Optional[DataRecord]:
+    def get_record(self, data: dict[str, Any]) -> DataRecord | None:
         """
         Retrieve idempotency key for data provided, fetch from persistence store, and convert to DataRecord.
 
         Parameters
         ----------
-        data: Dict[str, Any]
+        data: dict[str, Any]
             Payload
 
         Returns
@@ -449,14 +386,14 @@ class BasePersistenceLayer(ABC):
         cached_record = self._retrieve_from_cache(idempotency_key=idempotency_key)
         if cached_record:
             logger.debug(f"Idempotency record found in cache with idempotency key: {idempotency_key}")
-            self._validate_payload(data=data, data_record=cached_record)
+            self._validate_payload(data_payload=data, stored_data_record=cached_record)
             return cached_record
 
         record = self._get_record(idempotency_key=idempotency_key)
 
+        self._validate_payload(data_payload=data, stored_data_record=record)
         self._save_to_cache(data_record=record)
 
-        self._validate_payload(data=data, data_record=record)
         return record
 
     @abstractmethod
