@@ -1,5 +1,7 @@
+from __future__ import annotations
+
 from dataclasses import asdict, is_dataclass
-from typing import Any, Dict, Type
+from typing import Any
 
 from aws_lambda_powertools.utilities.idempotency.exceptions import (
     IdempotencyModelTypeError,
@@ -18,19 +20,19 @@ class DataclassSerializer(BaseIdempotencyModelSerializer):
     A serializer class for transforming data between dataclass objects and dictionaries.
     """
 
-    def __init__(self, model: Type[DataClass]):
+    def __init__(self, model: type[DataClass]):
         """
         Parameters
         ----------
-        model: Type[DataClass]
+        model: type[DataClass]
             A dataclass type to be used for serialization and deserialization
         """
-        self.__model: Type[DataClass] = model
+        self.__model: type[DataClass] = model
 
-    def to_dict(self, data: DataClass) -> Dict:
+    def to_dict(self, data: DataClass) -> dict:
         return asdict(data)
 
-    def from_dict(self, data: Dict) -> DataClass:
+    def from_dict(self, data: dict) -> DataClass:
         return self.__model(**data)
 
     @classmethod
@@ -40,4 +42,4 @@ class DataclassSerializer(BaseIdempotencyModelSerializer):
 
         if not is_dataclass(model_type):
             raise IdempotencyModelTypeError("Model type is not inherited of dataclass type")
-        return cls(model=model_type)
+        return cls(model=model_type)  # type: ignore[arg-type]
