@@ -1,14 +1,22 @@
+from __future__ import annotations
+
 import logging
 import typing
-from typing import Any, Callable, Dict, Optional, Type, overload
+from typing import TYPE_CHECKING, Any, Callable, overload
 
-from aws_lambda_powertools.utilities.parser.compat import disable_pydantic_v2_warning
-from aws_lambda_powertools.utilities.parser.types import EventParserReturnType, Model
+from pydantic import PydanticSchemaGenerationError
 
-from ...middleware_factory import lambda_handler_decorator
-from ..typing import LambdaContext
-from .envelopes.base import Envelope
-from .exceptions import InvalidEnvelopeError, InvalidModelTypeError
+from aws_lambda_powertools.middleware_factory import lambda_handler_decorator
+from aws_lambda_powertools.utilities.parser.exceptions import InvalidEnvelopeError, InvalidModelTypeError
+from aws_lambda_powertools.utilities.parser.functions import (
+    _parse_and_validate_event,
+    _retrieve_or_set_model_from_cache,
+)
+
+if TYPE_CHECKING:
+    from aws_lambda_powertools.utilities.parser.envelopes.base import Envelope
+    from aws_lambda_powertools.utilities.parser.types import EventParserReturnType, T
+    from aws_lambda_powertools.utilities.typing import LambdaContext
 
 logger = logging.getLogger(__name__)
 
@@ -16,10 +24,10 @@ logger = logging.getLogger(__name__)
 @lambda_handler_decorator
 def event_parser(
     handler: Callable[..., EventParserReturnType],
-    event: Dict[str, Any],
+    event: dict[str, Any],
     context: LambdaContext,
-    model: Optional[Type[Model]] = None,
-    envelope: Optional[Type[Envelope]] = None,
+    model: type[T] | None = None,
+    envelope: type[Envelope] | None = None,
     **kwargs: Any,
 ) -> EventParserReturnType:
     """Lambda handler decorator to parse & validate events using Pydantic models
@@ -34,7 +42,7 @@ def event_parser(
     This is useful when you need to confirm event wrapper structure, and
     b) selectively extract a portion of your payload for parsing & validation.
 
-    NOTE: If envelope is omitted, the complete event is parsed to match the model parameter BaseModel definition.
+    NOTE: If envelope is omitted, the complete event is parsed to match the model parameter definition.
 
     Example
     -------
@@ -64,11 +72,11 @@ def event_parser(
     ----------
     handler:  Callable
         Method to annotate on
-    event:    Dict
+    event:    dict
         Lambda event to be parsed & validated
     context:  LambdaContext
         Lambda context object
-    model:   Model
+    model:   type[T] | None
         Your data model that will replace the event.
     envelope: Envelope
         Optional envelope to extract the model from
@@ -83,37 +91,39 @@ def event_parser(
         When envelope given does not implement BaseEnvelope
     """
 
-    # The first parameter of a Lambda function is always the event
-    # This line get the model informed in the event_parser function
-    # or the first parameter of the function by using typing.get_type_hints
-    type_hints = typing.get_type_hints(handler)
-    model = model or (list(type_hints.values())[0] if type_hints else None)
     if model is None:
-        raise InvalidModelTypeError(
-            "The model must be provided either as the `model` argument to `event_parser`"
-            "or as the type hint of `event` in the handler that it wraps",
-        )
+        # The first parameter of a Lambda function is always the event.
+        # Get the first parameter's type by using typing.get_type_hints.
+        type_hints = typing.get_type_hints(handler)
+        if type_hints:
+            model = list(type_hints.values())[0]
+        if model is None:
+            raise InvalidModelTypeError(
+                "The model must be provided either as the `model` argument to `event_parser`"
+                "or as the type hint of `event` in the handler that it wraps",
+            )
 
-    if envelope:
-        parsed_event = parse(event=event, model=model, envelope=envelope)
-    else:
-        parsed_event = parse(event=event, model=model)
+    try:
+        if envelope:
+            parsed_event = parse(event=event, model=model, envelope=envelope)
+        else:
+            parsed_event = parse(event=event, model=model)
 
-    logger.debug(f"Calling handler {handler.__name__}")
-    return handler(parsed_event, context, **kwargs)
+        logger.debug(f"Calling handler {handler.__name__}")
+        return handler(parsed_event, context, **kwargs)
+    except AttributeError as exc:
+        raise InvalidModelTypeError(f"Error: {str(exc)}. Please ensure the type you're trying to parse into is correct")
 
 
 @overload
-def parse(event: Dict[str, Any], model: Type[Model]) -> Model:
-    ...  # pragma: no cover
+def parse(event: dict[str, Any], model: type[T]) -> T: ...  # pragma: no cover
 
 
 @overload
-def parse(event: Dict[str, Any], model: Type[Model], envelope: Type[Envelope]) -> Model:
-    ...  # pragma: no cover
+def parse(event: dict[str, Any], model: type[T], envelope: type[Envelope]) -> T: ...  # pragma: no cover
 
 
-def parse(event: Dict[str, Any], model: Type[Model], envelope: Optional[Type[Envelope]] = None):
+def parse(event: dict[str, Any], model: type[T], envelope: type[Envelope] | None = None):
     """Standalone function to parse & validate events using Pydantic models
 
     Typically used when you need fine-grained control over error handling compared to event_parser decorator.
@@ -151,7 +161,7 @@ def parse(event: Dict[str, Any], model: Type[Model], envelope: Optional[Type[Env
 
     Parameters
     ----------
-    event:    Dict
+    event:    dict
         Lambda event to be parsed & validated
     model:   Model
         Your data model that will replace the event
@@ -171,15 +181,27 @@ def parse(event: Dict[str, Any], model: Type[Model], envelope: Optional[Type[Env
         try:
             logger.debug(f"Parsing and validating event model with envelope={envelope}")
             return envelope().parse(data=event, model=model)
-        except AttributeError:
-            raise InvalidEnvelopeError(f"Envelope must implement BaseEnvelope, envelope={envelope}")
+        except AttributeError as exc:
+            raise InvalidEnvelopeError(
+                f"Error: {str(exc)}. Please ensure that both the Input model and the Envelope inherits from BaseModel,\n"  # noqa E501
+                "and your payload adheres to the specified Input model structure.\n"
+                f"Envelope={envelope}\nModel={model}",
+            ) from exc
 
     try:
-        disable_pydantic_v2_warning()
-        logger.debug("Parsing and validating event model; no envelope used")
-        if isinstance(event, str):
-            return model.parse_raw(event)
+        adapter = _retrieve_or_set_model_from_cache(model=model)
 
-        return model.parse_obj(event)
-    except AttributeError:
-        raise InvalidModelTypeError(f"Input model must implement BaseModel, model={model}")
+        logger.debug("Parsing and validating event model; no envelope used")
+        return _parse_and_validate_event(data=event, adapter=adapter)
+
+    # Pydantic raises PydanticSchemaGenerationError when the model is not a Pydantic model
+    # This is seen in the tests where we pass a non-Pydantic model type to the parser or
+    # when we pass a data structure that does not match the model (trying to parse a true/false/etc into a model)
+    except PydanticSchemaGenerationError as exc:
+        raise InvalidModelTypeError(f"The event supplied is unable to be validated into {type(model)}") from exc
+    except AttributeError as exc:
+        raise InvalidModelTypeError(
+            f"Error: {str(exc)}. Please ensure the Input model inherits from BaseModel,\n"
+            "and your payload adheres to the specified Input model structure.\n"
+            f"Model={model}",
+        ) from exc
