@@ -1,10 +1,11 @@
+from __future__ import annotations
+
 import logging
-from typing import Any, Callable, Dict, Optional, Union
+from typing import Any, Callable
 
+from aws_lambda_powertools.middleware_factory import lambda_handler_decorator
 from aws_lambda_powertools.utilities import jmespath_utils
-
-from ...middleware_factory import lambda_handler_decorator
-from .base import validate_data_against_schema
+from aws_lambda_powertools.utilities.validation.base import validate_data_against_schema
 
 logger = logging.getLogger(__name__)
 
@@ -12,14 +13,18 @@ logger = logging.getLogger(__name__)
 @lambda_handler_decorator
 def validator(
     handler: Callable,
-    event: Union[Dict, str],
+    event: dict | str,
     context: Any,
-    inbound_schema: Optional[Dict] = None,
-    inbound_formats: Optional[Dict] = None,
-    outbound_schema: Optional[Dict] = None,
-    outbound_formats: Optional[Dict] = None,
+    inbound_schema: dict | None = None,
+    inbound_formats: dict | None = None,
+    inbound_handlers: dict | None = None,
+    inbound_provider_options: dict | None = None,
+    outbound_schema: dict | None = None,
+    outbound_formats: dict | None = None,
+    outbound_handlers: dict | None = None,
+    outbound_provider_options: dict | None = None,
     envelope: str = "",
-    jmespath_options: Optional[Dict] = None,
+    jmespath_options: dict | None = None,
     **kwargs: Any,
 ) -> Any:
     """Lambda handler decorator to validate incoming/outbound data using a JSON Schema
@@ -28,22 +33,33 @@ def validator(
     ----------
     handler : Callable
         Method to annotate on
-    event : Dict
+    event : dict
         Lambda event to be validated
     context : Any
         Lambda context object
-    inbound_schema : Dict
+    inbound_schema : dict
         JSON Schema to validate incoming event
-    outbound_schema : Dict
+    outbound_schema : dict
         JSON Schema to validate outbound event
-    envelope : Dict
+    envelope : dict
         JMESPath expression to filter data against
-    jmespath_options : Dict
+    jmespath_options : dict
         Alternative JMESPath options to be included when filtering expr
-    inbound_formats: Dict
+    inbound_formats: dict
         Custom formats containing a key (e.g. int64) and a value expressed as regex or callback returning bool
-    outbound_formats: Dict
+    outbound_formats: dict
         Custom formats containing a key (e.g. int64) and a value expressed as regex or callback returning bool
+    inbound_handlers: Dict
+        Custom methods to retrieve remote schemes, keyed off of URI scheme
+    outbound_handlers: Dict
+        Custom methods to retrieve remote schemes, keyed off of URI scheme
+    inbound_provider_options: Dict
+        Arguments that will be passed directly to the underlying validation call, in this case fastjsonchema.validate.
+        For all supported arguments see: https://horejsek.github.io/python-fastjsonschema/#fastjsonschema.validate
+    outbound_provider_options: Dict
+        Arguments that will be passed directly to the underlying validation call, in this case fastjsonchema.validate.
+        For all supported arguments see: https://horejsek.github.io/python-fastjsonschema/#fastjsonschema.validate
+
 
     Example
     -------
@@ -119,7 +135,7 @@ def validator(
         When JMESPath expression to unwrap event is invalid
     """  # noqa: E501
     if envelope:
-        event = jmespath_utils.extract_data_from_envelope(
+        event = jmespath_utils.query(
             data=event,
             envelope=envelope,
             jmespath_options=jmespath_options,
@@ -127,40 +143,58 @@ def validator(
 
     if inbound_schema:
         logger.debug("Validating inbound event")
-        validate_data_against_schema(data=event, schema=inbound_schema, formats=inbound_formats)
+        validate_data_against_schema(
+            data=event,
+            schema=inbound_schema,
+            formats=inbound_formats,
+            handlers=inbound_handlers,
+            provider_options=inbound_provider_options,
+        )
 
     response = handler(event, context, **kwargs)
 
     if outbound_schema:
         logger.debug("Validating outbound event")
-        validate_data_against_schema(data=response, schema=outbound_schema, formats=outbound_formats)
+        validate_data_against_schema(
+            data=response,
+            schema=outbound_schema,
+            formats=outbound_formats,
+            handlers=outbound_handlers,
+            provider_options=outbound_provider_options,
+        )
 
     return response
 
 
 def validate(
     event: Any,
-    schema: Dict,
-    formats: Optional[Dict] = None,
-    envelope: Optional[str] = None,
-    jmespath_options: Optional[Dict] = None,
-):
+    schema: dict,
+    formats: dict | None = None,
+    handlers: dict | None = None,
+    provider_options: dict | None = None,
+    envelope: str | None = None,
+    jmespath_options: dict | None = None,
+) -> Any:
     """Standalone function to validate event data using a JSON Schema
 
      Typically used when you need more control over the validation process.
 
     Parameters
     ----------
-    event : Dict
+    event : dict
         Lambda event to be validated
-    schema : Dict
+    schema : dict
         JSON Schema to validate incoming event
-    envelope : Dict
+    envelope : dict
         JMESPath expression to filter data against
-    jmespath_options : Dict
+    jmespath_options : dict
         Alternative JMESPath options to be included when filtering expr
-    formats: Dict
+    formats: dict
         Custom formats containing a key (e.g. int64) and a value expressed as regex or callback returning bool
+    handlers: Dict
+        Custom methods to retrieve remote schemes, keyed off of URI scheme
+    provider_options: Dict
+        Arguments that will be passed directly to the underlying validate call
 
     Example
     -------
@@ -213,6 +247,12 @@ def validate(
             validate(event=event, schema=json_schema_dict, envelope="awslogs.powertools_base64_gzip(data) | powertools_json(@).logEvents[*]")
             return event
 
+    Returns
+    -------
+    Dict
+        The validated event. If the schema specifies a `default` value for fields that are omitted,
+        those default values will be included in the response.
+
     Raises
     ------
     SchemaValidationError
@@ -223,10 +263,16 @@ def validate(
         When JMESPath expression to unwrap event is invalid
     """  # noqa: E501
     if envelope:
-        event = jmespath_utils.extract_data_from_envelope(
+        event = jmespath_utils.query(
             data=event,
             envelope=envelope,
             jmespath_options=jmespath_options,
         )
 
-    validate_data_against_schema(data=event, schema=schema, formats=formats)
+    return validate_data_against_schema(
+        data=event,
+        schema=schema,
+        formats=formats,
+        handlers=handlers,
+        provider_options=provider_options,
+    )
